#!/usr/bin/env python
#Copyright (c) 2012 Roman Pollak <roman65536@yahoo.com>
#Copyright (c) 2011 Alan Aguiar, <alanjas@hotmail.com>
#Copyright (c) 2011 Aylen Ricca, <ar18_90@hotmail.com>
#Copyright (c) 2011 Rodrigo Dearmas, <piegrande46@hotmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gst
import gtk
from fcntl import ioctl
import sys, os
import logging
from gettext import gettext as _
from plugins.plugin import Plugin
from TurtleArt.tapalette import make_palette
from TurtleArt.talogo import media_blocks_dictionary, primitive_dictionary
from TurtleArt.taconstants import BLACK, WHITE, CONSTANTS, BOX_COLORS



COLOR_NOTPRESENT = ["#A0A0A0","#808080"]
_logger = logging.getLogger('turtleart-activity followme plugin')


def _num_type(x):
    """ Is x a number type? """
    if type(x) == int:
        return True
    if type(x) == float:
        return True
    if type(x) == ord:
        return True
    return False


class Logic(Plugin):

    def __init__(self, parent):
        self.parent = parent
        self.cam_present = False
        self.cam_on = False
        self.colorc = (255, 255, 255)
        self.threshold = (25, 25, 25)
        try:
            print _('The Logic was found.')
        except:
            print _('Error on the init Logic. ')


    def setup(self):
	print 'seting up'
        palette = make_palette('Logic', colors=["#FF00FF","#A000A0"],
                                help_string=_('Logic Operations'))

        primitive_dictionary['myxor'] = self.prim_xor
        palette.add_block('myxor',
                        style='number-style-porch',
                        label=[('XOR')],
                        help_string=_('Logical xor'),
                        prim_name='myxor')
        self.parent.lc.def_prim('myxor', 2, lambda self, x, y:
                        primitive_dictionary['myxor'](x, y))

        primitive_dictionary['myand'] = self.prim_myand
        palette.add_block('myand',
                        style='number-style-porch',
                        label=[('AND')],
                        help_string=_('Logical AND'),
                        prim_name='myand')
        self.parent.lc.def_prim('myand', 2, lambda self, x, y:
                        primitive_dictionary['myand'](x, y))

	primitive_dictionary['myor'] = self.prim_myor
        palette.add_block('myor',
                        style='number-style-porch',
                        label=[('OR')],
                        help_string=_('Logical OR'),
                        prim_name='myor')
        self.parent.lc.def_prim('myor', 2, lambda self, x, y:
                        primitive_dictionary['myor'](x, y))

        primitive_dictionary['mysl'] = self.prim_mysl
        palette.add_block('mysl',
                        style='number-style-porch',
                        label=[('<<')],
                        help_string=_('Logical Shift Left'),
                        prim_name='mysl')
        self.parent.lc.def_prim('mysl', 2, lambda self, x, y:
                        primitive_dictionary['mysl'](x, y))

        primitive_dictionary['mysr'] = self.prim_mysr
        palette.add_block('mysr',
                        style='number-style-porch',
                        label=[('>>')],
                        help_string=_('Logical Shift Right'),
                        prim_name='mysr')
        self.parent.lc.def_prim('mysr', 2, lambda self, x, y:
                        primitive_dictionary['mysr'](x, y))

        primitive_dictionary['mynot'] = self.prim_mynot
        palette.add_block('mynot',
                        style='number-style-1arg',
                        label=[('NOT')],
                        help_string=_('Logical NOT'),
                        prim_name='mynot')
        self.parent.lc.def_prim('mynot', 1, lambda self, x:
                        primitive_dictionary['mynot'](x))



	print 'done'
	


    def stop(self):
	print 'stoped'

    def quit(self):
	print 'quit'


    def prim_xor(self, x,  y):
	""" Numerical mod """
        if _num_type(x) and _num_type(y):
            return(int(x) ^ int(y))
        try:
            return self._string_to_num(x) ^ self._string_to_num(y)
        except TypeError:
            raise logoerror("#notanumber")
        except ValueError:
            raise logoerror("#syntaxerror")

    def prim_myand(self, x,  y):
	""" Numerical mod """
        if _num_type(x) and _num_type(y):
            return(int(x) & int(y))
        try:
            return self._string_to_num(x) & self._string_to_num(y)
        except TypeError:
            raise logoerror("#notanumber")
        except ValueError:
            raise logoerror("#syntaxerror")

    def prim_myor(self, x,  y):
        """ Numerical mod """
        if _num_type(x) and _num_type(y):
            return(int(x) | int(y))
        try:
            return self._string_to_num(x) | self._string_to_num(y)
        except TypeError:
            raise logoerror("#notanumber")
        except ValueError:
            raise logoerror("#syntaxerror")

    def prim_mysl(self, x,  y):
        """ Numerical mod """
        if _num_type(x) and _num_type(y):
            return(int(x) << int(y))
        try:
            return self._string_to_num(x) | self._string_to_num(y)
        except TypeError:
            raise logoerror("#notanumber")
        except ValueError:
            raise logoerror("#syntaxerror")

    def prim_mysr(self, x,  y):
        """ Numerical mod """
        if _num_type(x) and _num_type(y):
            return(int(x) >> int(y))
        try:
            return self._string_to_num(x) | self._string_to_num(y)
        except TypeError:
            raise logoerror("#notanumber")
        except ValueError:
            raise logoerror("#syntaxerror")

    def prim_mynot(self, x):
        """ Numerical mod """
        if _num_type(x):
            return(~ int(x) )
        try:
            return self._string_to_num(x) | self._string_to_num(y)
        except TypeError:
            raise logoerror("#notanumber")
        except ValueError:
            raise logoerror("#syntaxerror")




